#!/usr/bin/env pythonw
"""Ground station software for FreedomFlies, an opensource UAV.
Surveil the surveillers!

Written by Josh Levinger, Fall 2005

Licensed under the MIT License
--------------------------------------------------------------
Copyright (c) 2005 MIT Media Lab

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
--------------------------------------------------------------
Brought to you by: Willcox Playa AZ, Home Depot, Busch Beer and Sex Musk
"""

#main imports
import time
import math
import sys,os
import threading

#my imports
import prefs
import joystick
import compass
import horizon
import log
from bufferedcanvas import *
import datalink

#external library imports
try:
	import wx
	from wx.lib.dialogs import * #for about box
except ImportError:
	print "cannot import wx"
try:
	import pygame
except ImportError:
	print "cannot import pygame"
try:
	import serial
except ImportError:
	print "cannot import pyserial"


class AppFrame(wx.Frame):
	def __init__(self,*args,**kwds):
		wx.Frame.__init__(self,*args,**kwds)

		#top sizer elements
		self.airspeed_gauge = wx.Gauge(self, -1, 100, size=(20,150), style=wx.GA_VERTICAL|wx.GA_SMOOTH)
		self.altitude_gauge = wx.Gauge(self, -1, 100, size=(20,150), style=wx.GA_VERTICAL|wx.GA_SMOOTH)
		self.horizon = horizon.MyHorizonIndicator(self,-1,(500,150))
		self.airspeed_value = wx.TextCtrl(self,-1,size=(50,20),style=wx.TE_READONLY)
		self.altitude_value = wx.TextCtrl(self,-1,size=(50,20),style=wx.TE_READONLY)
	
		#bottom sizer elements
		button_width = (80,20)
		self.autopilot_button = wx.ToggleButton(self, -1, "Autopilot",size=button_width)
		self.radio_button = wx.ToggleButton(self, -1, "Radio",size=button_width)
		self.engine_button = wx.ToggleButton(self, -1, "Engine",size=button_width)
		self.logfile_button = wx.ToggleButton(self, -1, "Log to file",size=button_width)
		self.button_1 = wx.ToggleButton(self, -1, "",size=button_width)
		self.button_2 = wx.ToggleButton(self, -1, "",size=button_width)
		
		self.notebook = wx.Notebook(self, -1, size = (200,40), style=0)
		self.error_log_ctrl = wx.TextCtrl(self.notebook, -1, style = wx.TE_MULTILINE|wx.TE_READONLY)
		self.downlink_log_ctrl = wx.TextCtrl(self.notebook, -1, style = wx.TE_MULTILINE|wx.TE_READONLY)
		self.uplink_log_ctrl = wx.TextCtrl(self.notebook, -1, style = wx.TE_MULTILINE|wx.TE_READONLY)
		
		log.SetLogs(self.error_log_ctrl,
		self.downlink_log_ctrl,
		self.uplink_log_ctrl) #send textctrls to log.py
		
		
		self.lat_deg_ctrl = wx.TextCtrl(self, -1,size=(30,20),style=wx.TE_READONLY|wx.TE_RICH2)
		self.lat_min_ctrl = wx.TextCtrl(self, -1,size=(60,20),style=wx.TE_READONLY|wx.TE_RICH2)
		self.lat_dir_text = wx.StaticText(self, -1, " ")
		self.lon_deg_ctrl = wx.TextCtrl(self, -1,size=(30,20),style=wx.TE_READONLY|wx.TE_RICH2)
		self.lon_min_ctrl = wx.TextCtrl(self, -1,size=(60,20),style=wx.TE_READONLY|wx.TE_RICH2)
		self.lon_dir_text = wx.StaticText(self, -1, " ")
		self.gps_error_ctrl = wx.TextCtrl(self, -1,size=(100,20),style=wx.TE_READONLY|wx.TE_RICH2)
		self.throttle_gauge = wx.Gauge(self, -1, 10,size=(100,20),style=wx.GA_HORIZONTAL|wx.GA_SMOOTH)

		self.compass = compass.MyCompass(self,-1,(250,250))

		main_box_sizer = wx.BoxSizer(wx.HORIZONTAL)
		main_grid_sizer = wx.GridSizer(2, 1)
	
		top_sizer = wx.BoxSizer(wx.HORIZONTAL)
		bottom_sizer = wx.FlexGridSizer(1, 3,hgap=5)
		
		airspeed_sizer = wx.BoxSizer(wx.HORIZONTAL)
		altitude_sizer = wx.BoxSizer(wx.HORIZONTAL)
		airspeed_sizer.Add(self.airspeed_value,0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT,3)
		airspeed_sizer.Add(self.airspeed_gauge,0,wx.RIGHT,-2)
		altitude_sizer.Add(self.altitude_gauge,0,wx.LEFT,5)
		altitude_sizer.Add(self.altitude_value,0,wx.ALIGN_CENTER_VERTICAL|wx.RIGHT,0)
		
		top_sizer.Add(airspeed_sizer, 0)
		top_sizer.Add(self.horizon, 0,wx.LEFT)
		top_sizer.Add(altitude_sizer, 0)
		main_grid_sizer.Add(top_sizer,1,wx.LEFT,0)
		
		button_sizer = wx.GridSizer(4, 2, vgap=10,hgap=10)
		button_sizer.Add(self.radio_button, 0)
		button_sizer.Add(self.autopilot_button, 0)
		button_sizer.Add(self.engine_button, 0)
		button_sizer.Add(self.logfile_button, 0)
		button_sizer.Add(self.button_1, 0)
		button_sizer.Add(self.button_2, 0)
		bottom_sizer.Add(button_sizer,1,wx.LEFT|wx.ALIGN_CENTER_VERTICAL,5)
		wx.EVT_TOGGLEBUTTON(self,self.radio_button.GetId(),self.OnRadioButton)
		
		self.notebook.AddPage(self.error_log_ctrl, "Error")
		self.notebook.AddPage(self.downlink_log_ctrl, "Downlink")
		self.notebook.AddPage(self.uplink_log_ctrl, "Uplink")
		bottom_sizer.Add(self.notebook,1)
		
		info_sizer = wx.FlexGridSizer(4,2,vgap=5)
		lat_sizer = wx.BoxSizer(wx.HORIZONTAL)
		info_sizer.Add(wx.StaticText(self, -1, "Latitude"),1)
		lat_sizer.Add(self.lat_deg_ctrl,0,wx.RIGHT,5)
		lat_sizer.Add(self.lat_min_ctrl,0,wx.RIGHT,5)
		lat_sizer.Add(self.lat_dir_text,0)
		info_sizer.Add(lat_sizer,0,wx.ALIGN_LEFT,0)
		lon_sizer = wx.BoxSizer(wx.HORIZONTAL)
		info_sizer.Add(wx.StaticText(self, -1, "Longitude"),1)
		lon_sizer.Add(self.lon_deg_ctrl,0,wx.RIGHT,5)
		lon_sizer.Add(self.lon_min_ctrl,0,wx.RIGHT,5)
		lon_sizer.Add(self.lon_dir_text,0)
		info_sizer.Add(lon_sizer,0,wx.ALIGN_LEFT,0)
		
		info_sizer.Add(wx.StaticText(self, -1, "Accuracy"),1)
		info_sizer.Add(self.gps_error_ctrl)
		info_sizer.Add(wx.StaticText(self, -1, "Throttle"),1)
		info_sizer.Add(self.throttle_gauge)
		bottom_sizer.Add(info_sizer, 0, wx.ALIGN_CENTER_VERTICAL|wx.RIGHT)
		
		compass_sizer = wx.BoxSizer(wx.HORIZONTAL)
		compass_sizer.Add(self.compass,0)
		
		main_grid_sizer.Add(bottom_sizer,1,wx.LEFT,0)
		main_box_sizer.Add(main_grid_sizer,1,wx.LEFT,0)
		main_box_sizer.Add(compass_sizer,0,wx.TOP|wx.RIGHT|wx.LEFT,5)
		self.SetMinSize((920,300))
		self.SetAutoLayout(True)
		self.SetSizer(main_box_sizer)
		self.Layout()
		#end of main window items
		
		#Menubar
		self.MenuBar = wx.MenuBar()
		FileMenu = wx.Menu()
		FileMenu.Append(-1, 'Save log')
		FileMenu.AppendSeparator()
		prefs = FileMenu.Append(-1,'Preferences')		
		quit = FileMenu.Append(-1, '&Quit')
		wx.EVT_MENU(self,quit.GetId(), self.OnQuit)
		if "__WXMAC__" in wx.PlatformInfo: 
			wx.App_SetMacExitMenuItemId(quit.GetId())
			#wx.App_SetMacPreferencesMenuItemId(prefs.GetId()) #why doesn't this work?
		self.MenuBar.Append(FileMenu, '&File')

		HelpMenu = wx.Menu()
		help = HelpMenu.Append(-1, 'About FreedomFlies')
		wx.EVT_MENU(self, help.GetId(), self.OnAbout)
		wx.EVT_MENU(self, prefs.GetId(),self.OnPrefs)
		if "__WXMAC__" in wx.PlatformInfo: 
			wx.App.SetMacAboutMenuItemId(help.GetId())
			wx.App_SetMacHelpMenuTitleName("&Help")
		self.MenuBar.Append(HelpMenu, "&Help")
		
		SetupMenu = wx.Menu()
		cal = SetupMenu.Append(-1,'Calibrate Joystick')
		wx.EVT_MENU(self,cal.GetId(),self.OnJoystickCalibrate)
		gtest = SetupMenu.Append(-1,"Graphics Test")
		wx.EVT_MENU(self,gtest.GetId(),self.OnGraphicsTest)
		self.MenuBar.Append(SetupMenu,"Setup")
		self.SetMenuBar(self.MenuBar)
		#end of menubar
		
		#global objects 
		self.gpsout_port = ""
		self.radio_port = ""
		self.joystickCalibrated = False
		self.stik = joystick.Joystick()	
		#startup
		self.OnPrefs(None)
		self.OnJoystickCalibrate(None)
	#end __init__
		
	def OnQuit(self, event):
		self.Close()
		sys.exit()
	def OnAbout(self, event):
		about = ScrolledMessageDialog(self, __doc__, "About...")
		about.ShowModal()
	def OnPrefs(self,event):
		p = prefs.PrefFrame(self,-1,"Preferences",(50,325),(300,150))
		p.Show()
	def OnGraphicsTest(self,event):
		for i in range(0,361,2):
			self.compass.SetHeading(i)
			wx.Yield()
	def OnJoystickCalibrate(self,event):
		self.calib = joystick.JoyFrame(self,-1,"Joystick Calibration",(955,30),(200,225))
		self.calib.Layout()
		self.calib.Show()
	def OnRadioButton(self,event):
		btn = event.GetEventObject()
		if btn.GetValue() == True:
			if self.joystickCalibrated is False:
				pass
				#log.Log('e',"Calibrate Joystick")
				#self.OnJoystickCalibrate(None)
			if self.radio_port == "":
				log.Log('e',"No port set in preferences")
			try:
				#radio_port, gpsout_port are set by prefs dialog
				black = wx.TextAttr(wx.BLACK)
				self.lat_deg_ctrl.SetDefaultStyle(black)
				self.lat_min_ctrl.SetDefaultStyle(black)
				self.lon_deg_ctrl.SetDefaultStyle(black)
				self.lon_min_ctrl.SetDefaultStyle(black)
				#self.accuracy_ctrl.SetDefaultStyle(black)
				self.radio = datalink.radiolink(self,self.radio_port,self.gpsout_port)
				self.radio.StartThread()
			except serial.SerialException,e:
				log.Log('e',str(e))
		if btn.GetValue() == False:
			try:		
				self.radio.StopThread()
				red = wx.TextAttr(wx.RED)
				self.lat_deg_ctrl.SetDefaultStyle(red)
				self.lat_min_ctrl.SetDefaultStyle(red)
				self.lon_deg_ctrl.SetDefaultStyle(red)
				self.lon_min_ctrl.SetDefaultStyle(red)
				#self.accuracy_ctrl.SetDefaultStyle(red)
				#set old data to red
			except AttributeError:
				#gps object doesn't exist yet, user clicked before setting value in prefs
				pass
	def UpdatePositionInfo(self,lat_deg,lat_min,lat_dir,lon_deg,lon_min,lon_dir):
		self.lat_deg_ctrl.SetValue(lat_deg)
		self.lat_min_ctrl.SetValue(lat_min)
		self.lat_dir_text.SetLabel(lat_dir)
		self.lon_deg_ctrl.SetValue(lon_deg)
		self.lon_min_ctrl.SetValue(lon_min)
		self.lon_dir_text.SetLabel(lon_dir)
	def UpdateAltitude(self,alt):
		self.altitude_value.SetValue(alt)
		#self.altitude_gauge.SetValue(alt/100)
	def UpdateAirspeed(self,v):
		self.airspeed_value.SetValue(v)
		#self.airspeed_gauge.SetValue(v/100)
# end of class AppFrame

class MyApp(wx.App):
	def OnInit(self):
		#pygame.init()
		wx.InitAllImageHandlers()
		self.win = AppFrame(None, -1, "Freedom Flies",(50,25),(920,300))
		self.win.Layout()
		self.win.Show()
		self.SetTopWindow(self.win)
		self.Bind(wx.EVT_CLOSE,self.OnExit)
		return True
		
	def OnExit(self):
		pygame.quit()	
# end of class MyApp

if __name__ == "__main__":
	app = MyApp(0)
	app.MainLoop()
